<?php
/*
Plugin Name: wp-tmkm-amazon
Plugin URI: http://tomokame.moo.jp/
Description: ASIN を指定して Amazon から個別商品の情報を取出します。BOOKS, DVD, CD は詳細情報を取り出せます。
Author: ともかめ（Customized by Hametuha）
Version: 2.01ja alpha
Author URI: http://tomokame.moo.jp/
Special Thanks: Keith Devens.com (http://keithdevens.com/software/phpxml)
Special Thanks: websitepublisher.net (http://www.websitepublisher.net/article/aws-php/)
Special Thanks: hiromasa.zone :o) (http://hiromasa.zone.ne.jp/)
Special Thanks: PEAR :: Package :: Cache_Lite (http://pear.php.net/package/Cache_Lite)
*/

/********** Notes
 # ECS4.0 に対応しています。
 # PHP4.x で動作します。ただし Keith Devens.com の PHP XML Library が必要（ 同梱しています ）。
 # [tmkm-amazon]ASIN[/tmkm-amazon] または  tmkm_amazon_view('ASIN');  という記述で動作します。
 # LGPL で提供されている Lite.php および Open Source License で提供されている xml.php を同梱しています。
 # 記事およびページ投稿／編集画面での Amazon 検索が可能です。
**********/

/********** Usage
 # 1. ダウンロードした zip ファイルを解凍します。
 # 2. wp-tmkm-amazon フォルダを wp-content/plugins フォルダに転送します。
 # 3. 管理画面から wp-tmkm-amazon を有効化します。
 # 4. 管理画面にある「設定」画面内の「Wp_Tmkm_Amazon」メニューで、ご自分のアソシエイト ID を入力します。
      そのほか管理画面では、リンクウィンドウの挙動や詳細表示を設定できます。
 # 5. 各テーマの php ファイル、もしくは記事本文中に以下を記載します。
 #### PHP 関数として呼び出す場合	...	 tmkm_amazon_view('ASIN'); 　：テーマファイルに記述
 #### 記事本文中にコードを書く場合	...	[tmkm-amazon]ASIN[/tmkm-amazon]
 
 ## 記事本文中で PHP コードを実行できるプラグインを導入していれば、PHP 関数として呼び出すこともできます。
 ## 書籍の場合、ASIN に 10 桁および 13 桁の ISBN を使用できます。
 ## 同梱の amazon_noimg.png と amazon_noimg_small.png を差し替えれば、商品画像がないときの代替画像を好きなものにできます。
**********/

/******************************************************************************
 * wp-tmkm-amazon-function : amazon ECS search engine
 *****************************************************************************/
if( basename( $_SERVER['SCRIPT_FILENAME'] ) == 'wp-tmkm-amazon.php' ) {
	die();
}

/**
 * @var string
 */
$tmkm_plugin_directory = plugin_dir_url(__FILE__);

/**
 * @var string
 */
$tmkm_amazon_php = '/wp-tmkm-amazon.php';

/**
 * @var string
 */
$tmkm_amazon_search_php = 'wp-tmkm-amazon-search.php';

/**
 * @var array Default Options
 */
$_tmkm_amazon_settings_default = array(
	'associatesid' => 'tomokametei-22',
	'accessKey' => '',
	'secretKey' => '',
	'windowtarget' => 'self',
	'goodsimgsize' => 'small',
	'layout_type' => '0'
);

/**
 * Global Setting for tmkm Amazon
 * @var array
 */
$tmkm_amazon_settings = get_option('wp_tmkm_admin_options', $_tmkm_amazon_settings_default);
foreach($_tmkm_amazon_settings_default as $key => $val){
	if(!isset($tmkm_amazon_settings[$key])){
		$tmkm_amazon_settings[$key] = $val;
	}
}

//インクルードパスの設定
set_include_path(dirname(__FILE__).'/pear'. PATH_SEPARATOR . get_include_path());

/******************************************************************************
 * THIS FILE IS CALLED ONLY.
 *****************************************************************************/
include_once dirname(__FILE__).DIRECTORY_SEPARATOR.'wp-tmkm-amazon-function.php';
$wpTmkmAmazonParser = new WpTmkmAmazonParser($tmkm_amazon_settings['accessKey'], $tmkm_amazon_settings['secretKey'], $tmkm_amazon_settings['associatesid']);



/**
 * Controller for Admin Panel
 */
class WpTmkmAmazonAdmin {
	
	/**
	 * Page slug on admin panel
	 * @var string
	 */
	var $slug  = 'wp-tmkm-amazon';
	
	/**
	 * Message Container
	 * @var array
	 */
	var $message = array('error' => array(), 'normal' => array());
	
	/**
	 * Constructor
	 * @return void
	 */
	function __construct(){
		//Add Options Page
		add_action('admin_menu', array($this, 'add_menu'));
		//Action Hook for admin_init
		add_action('admin_init', array($this, 'admin_init'));
	}

	/**
	 * Add Options Page
	 * @global string $tmkm_amazon_php 
	 */
	function add_menu() {
		// Add a new menu under Options:
		add_options_page(
			'WP tmkm Amazon',
			'WP tmkm Amazon',
			8,
			$this->slug,
			array($this, 'options_page')
		);
	}
	
	/**
	 * Admin init hook.
	 */
	function admin_init(){
		//Add metabox.
		foreach(array('post', 'page') as $post_type){
			add_meta_box('wp_tmkm_amazon_search', 'WP tmkm Amazon商品検索', array($this, 'metabox'), $post_type, 'advanced', 'high');
		}
	}
	
	/**
	 * Create Admin Panel
	 * @global type $tmkm_amazon_settings 
	 * @return void
	 */
	function options_page() {
		global $tmkm_amazon_settings;

		//Save options.
		if (isset($_POST['_wpnonce']) && wp_verify_nonce($_POST['_wpnonce'], 'wp_tmkm_amazon_setting')){
			$tmkm_amazon_settings = array(
				'associatesid' => htmlspecialchars($_POST['associatesid']),
				'accessKey' => (string)$_POST['accessKey'],
				'secretKey' => (string)$_POST['secretKey'],
				'windowtarget' => (string)$_POST['windowtarget'],
				'goodsimgsize' => (string)$_POST['goodsimgsize'],
				'layout_type' => intval($_POST['layout_type'])
			);
			update_option('wp_tmkm_admin_options', $tmkm_amazon_settings);
			$this->message['normal'][] = '<strong>設定を保存しました。</strong>'; 
		}
		
		//Check if all option is saved.
		if(empty($tmkm_amazon_settings['associatesid']) || $tmkm_amazon_settings['associatesid'] == 'tomokametei-22'){
			$this->message['error'][] = 'アソシエイトIDガ入力されていません。<a href="https://affiliate.amazon.co.jp/" target="_blank">Amazonアソシエイト</a>でアソシエイトIDを取得してください。';
		}
		if(empty($tmkm_amazon_settings['accessKey']) || empty($tmkm_amazon_settings['secretKey'])){
			$this->message['error'][] = 'アクセスIDまたはシークレットアクセスIDが入力されていません。<a href="http://aws.amazon.com/jp/" target="_blank">Amazon Web Service</a>に登録し、アクセス認証情報を取得してください。';
		}
		
		add_action('admin_notice', array($this, 'show_message'));

		switch( $tmkm_amazon_settings['windowtarget']) {
			case 'newwin':$newwindow = ' checked';$selfwindow = '';break;
			case 'self':$newwindow = '';$selfwindow = ' checked';break;
		}

		switch( $tmkm_amazon_settings['goodsimgsize'] ) {
			case 'medium': $m_goodssize = ' checked'; $s_goodssize = ''; break;
			case 'small': $m_goodssize = ''; $s_goodssize = ' checked'; break;
		}

		switch( $tmkm_amazon_settings['layout_type'] ) {
			case 0: $default_layout  = ' checked';$medium_layout = '';  $simple_layout = ''; $noimage_layout = ''; break;
			case 1: $default_layout = ''; $medium_layout = ' checked'; $simple_layout = ''; $noimage_layout = ''; break;
			case 2: $default_layout  = ''; $medium_layout = ''; $simple_layout = ' checked'; $noimage_layout = ''; break;
			case 3: $default_layout  = ''; $medium_layout = ''; $simple_layout = ''; $noimage_layout = ' checked'; break;
		}

		?>
			<div class="wrap" id="footnote-options">
			<h2>Wp Tmkm Amazon プラグイン設定</h2>
			<?php do_action('admin_notice'); ?>
			<form method="post">
				<?php wp_nonce_field('wp_tmkm_amazon_setting'); ?>
				<input type="hidden" name="action" value="save_options" />
				<table class="form-table">
					<tr>
						<th><label for="associatesid">あなたのアソシエイト ID</label></th>
						<td>
							<input type="text" class="regular-text" id="associatesid" name="associatesid" value="<?php echo $tmkm_amazon_settings['associatesid']; ?>" />
						</td>
					</tr>
					<tr>
						<th><label for="associatesid">あなたのAWS アクセス ID</label></th>
						<td>
							<input type="text" class="regular-text" id="accessKey" name="accessKey" value="<?php echo $tmkm_amazon_settings['accessKey']; ?>" />
						</td>
					</tr>
					<tr>
						<th><label for="associatesid">あなたのAWS シークレットアクセス ID</label></th>
						<td>
							<input type="text" class="regular-text" id="secretKey" name="secretKey" value="<?php echo $tmkm_amazon_settings['secretKey']; ?>" />
						</td>
					</tr>
					<tr>
						<th>商品リンクの動作</th>
						<td>
							<label>
								<input type="radio" name="windowtarget" value="self"<?php echo $selfwindow; ?> />
								&nbsp;同じウィンドウ（ target 指定なし ）
							</label><br />
							<label>
								<input type="radio" name="windowtarget" value="newwin"<?php echo $newwindow; ?> />
								&nbsp;新規ウィンドウ（ target="_blank" ）
							</label>
						</td>
					</tr>
					<tr>
						<th>商品詳細の表示スタイル</th>
						<td>
							<label>
								<input type="radio" name="layout_type" value="0"<?php echo $default_layout; ?> />
								&nbsp;画像、タイトル、出版社、発売時期、著者、価格、本のタイプ、ページ数、ISBN（ 初期設定。本以外はこれに準ずる項目 ）
							</label><br />
							<label>
								<input type="radio" name="layout_type" value="1"<?php echo $medium_layout; ?> />
								&nbsp;画像、タイトル、出版社、著者、発売時期（ 初期設定から価格情報とコード情報を省略 ）<br />
							</label><br />
							<label>
								<input type="radio" name="layout_type" value="2"<?php echo $simple_layout; ?> />
								&nbsp;画像とタイトルのみ<br />
							</label><br />
							<label>
								<input type="radio" name="layout_type" value="3"<?php echo $noimage_layout; ?> />
								&nbsp;タイトルのみ
							</label>
						</td>
					</tr>
					<tr>
						<th>商品画像サイズ</th>
						<td>
							<label>
								<input type="radio" name="goodsimgsize" value="small"<?php echo $s_goodssize; ?> />
								&nbsp;小サイズ（ 初期設定 ）
							</label><br />
							<label>
								<input type="radio" name="goodsimgsize" value="medium"<?php echo $m_goodssize ; ?> />
								&nbsp;中サイズ
							</label>
						</td>
					</tr>
				</table>
				<?php submit_button('設定を保存する')?>
			</form>
			</div>
		<?php
	}
	
	/**
	 * Create Meta box on admin panel
	 * @global $tmkm_plugin_directory
	 * @return void
	 */
	function metabox($post, $metabox){
		global $tmkm_plugin_directory, $tmkm_amazon_search_php;
		?>
		<p>iframe 内に検索フォームが表示されない方は<a href="<?php echo $tmkm_plugin_directory.$tmkm_amazon_search_php; ?>" target="_blank">別ウィンドウ</a>で検索をお願いします。</p>
			<iframe id="tmkm-iframe" style="width:100%" frameborder="0" src="<?php echo $tmkm_plugin_directory.$tmkm_amazon_search_php; ?>">IFRAME による表示がサポートされている環境が必要です。</iframe>
		<?php
	}
	
	/**
	 * Echo Message on admin panel
	 */
	function show_message(){
		if(!empty($this->message['normal'])){
			?>
				<div class="updated">
					<?php foreach($this->message['normal'] as $message): ?>
						<p><?php echo $message; ?></p>
					<?php endforeach; ?>
				</div>
			<?php
		}
		if(!empty($this->message['error'])){
			?>
				<div class="error">
					<?php foreach($this->message['error'] as $message): ?>
						<p><?php echo $message; ?></p>
					<?php endforeach; ?>
				</div>
			<?php
		}
	}
}


/**
 * ASINから商品を出力するコントローラー
 * 
 */
class WpTmkmAmazonList {

	/**
	 * Constructor
	 */
	function __construct() {
		add_shortcode('tmkm-amazon-list', array ($this, 'tmkm_amazon_list'));
		add_shortcode('tmkm-amazon', array($this, 'replace_strings'));
	}
	
	/**
	 * 記事本文中のショートコードを個別商品表示 HTML に置き換える
	 * 
	 * @param $content
	 * @return $transformedstring
	 */
	function replace_strings($atts, $content = null) {
		$asin = $content;
		$display = $this->format_amazon($asin);
		return $display;
	}
	
	/**
	 * PHP 関数として Amazon の個別商品 HTML を呼び出す
	 * 
	 * @param $asin ( ASIN )
	 * @return echo $display ( HTML )
	 */
	function amazon_view( $asin ) {
		$display = $this->format_amazon( $asin);
		echo $display;
	}
	
	/**
	 * Create HTML Source With Asin
	 * @global array $tmkm_amazon_settings
	 * @global string $tmkm_plugin_directory
	 * @global WpTmkmAmazonParser $wpTmkmAmazonParser
	 * @param string $asin
	 * @return string 
	 */
	function format_amazon($asin) {
		global $tmkm_amazon_settings, $tmkm_plugin_directory, $wpTmkmAmazonParser;
		$output = '';
        switch( $tmkm_amazon_settings['windowtarget']) {
        	case 'newwin': $windowtarget = ' target="_blank"'; break;
        	case 'self': $windowtarget = '';
        }
		
		$result = $wpTmkmAmazonParser->get_itme_by_asin($asin);

		if(is_wp_error($result)){ 
			//// Amazon function was returned false, so AWS is down
			$output = '<p>アマゾンのサーバでエラーが起こっているかもしれません。一度ページを再読み込みしてみてください。</p>';
		}else{
			// Amazon function returned XML data
			$status = $result->Items->Request->IsValid;
			if( $status == 'False' ){
				// Request is invalid
				$output = '<p>与えられたリクエストが正しくありません</p>';
			}else{
				// results were found, so display the products
				$item = $result->Items->Item[0];
				$atts = $wpTmkmAmazonParser->get_atts($item);
				$smallimage = $wpTmkmAmazonParser->get_image_src($item, 'small');
				$mediumimage = $wpTmkmAmazonParser->get_image_src($item, 'medium');

				$url = $item->DetailPageURL;
				$Title = $atts['Title'];
				$ProductGroup = isset($wpTmkmAmazonParser->searchIndex[$atts['ProductGroup']]) ? $wpTmkmAmazonParser->searchIndex[$atts['ProductGroup']]: '不明' ;
				if(isset($atts['ProductGroup']) && $atts['ProductGroup'] == 'Book'){
					$ProductGroup = '書籍';
				}
				$ProductGroup = " <small>[{$ProductGroup}]</small>";
				$price = $atts['ListPrice']['FormattedPrice'];
				

				if( $tmkm_amazon_settings['layout_type'] != '3' ) {
			        switch( $tmkm_amazon_settings['goodsimgsize'] ) {
			        	case 'small': $goodsimage = $smallimage; break;
			        	case 'medium': $goodsimage = $mediumimage; break;
			        }
				}
				$desc = $price ? "<p><em>価格: </em>{$price}</p>" : '';
				$filter = array(
					'author' => array('Author', 'Director', 'Actor', 'Artist', 'Creator'),
					'publisher' => array('Publisher', 'Studio', 'Label', 'Brand', 'Manufacturer'),
					'Date' => array('PublicationDate'),
					'allowable' => array('Binding', 'NumberOfPages', 'ISBN', 'Feature')
				);
				foreach($filter as $filter => $vals){
					foreach($vals as $val){
						if(isset($atts[$val])){
							$key = $wpTmkmAmazonParser->atts_to_string($val);
							$desc .= "<p><em>{$key}: </em>{$atts[$val]}</p>";
							if($filter != 'allowable' && $filter != 'author'){
								break;
							}
						}
					}
				}
				$output = <<<EOS
<div class="tmkm-amazon-view">
<p><a href="{$url}"{$windowtarget}><img src="{$mediumimage}" border="0" alt="{$Title}" /></a></p>
<p><a href="{$url}"{$windowtarget}>{$Title}{$ProductGroup}</a></p>
{$desc}
<hr class="tmkm-amazon-clear" />
</div>
EOS;
			}
		}
		return $output;
	}

	/**
	 * Returns Amazon List
	 * @global wpdb $wpdb
	 * @global array $tmkm_amazon_settings
	 * @param array $attr
	 * @return string 
	 */
	function tmkm_amazon_list( $attr ) {
		
		global $wpdb, $tmkm_amazon_settings;

		extract( shortcode_atts( array(
			'orderby' 	=> 'post_id',
			'order'		=> 'asc',
		), $attr ));

		$orderby = strval( $orderby );
		$order = strtoupper(strval( $order ));

		$output = '';

		switch( $orderby ) {
			case 'post_id': $ordersql = "ID " . $order; break;
			case 'post_title': $ordersql = "post_title " . $order; break;
			case 'modified_date': $ordersql = "post_modified " . $order; break;
			default: $ordersql = "post_date " . $order; break;
		}

	    $sql = <<<EOS
			SELECT SQL_CALC_FOUND_ROWS
				ID, post_title, post_date, post_content
			FROM
				{$wpdb->posts}
			WHERE
				post_status = 'publish' AND
				post_content LIKE '%[tmkm-amazon]%'
			ORDER BY
				$ordersql
EOS;

	    $PostRetainAsin = $wpdb->get_results($sql);
	    $postcount = $wpdb->get_var('SELECT FOUND_ROWS()');
	    $perpage = get_option("posts_per_page");
	    if( $PostRetainAsin ) {
			$heredoc = '';
			$books = array();
	    	foreach( $PostRetainAsin as $asinlist ) {
				$matches = array();
				if(!preg_match_all("/\[tmkm-amazon\]([0-9]+)\[\/tmkm-amazon\]/", $asinlist->post_content, $matches)){
					continue;
				}
	    		$permalink = get_permalink($asinlist->ID);
				$date = mysql2date(get_option('date_format'), $asinlist->post_date, false);
				$asins = $matches[1];
	    		foreach( $asins as $asin ) {
	    			$display = $this->format_amazon( $asin);
					$tag = '<p class="tmkm-amazon-clear"><em><em></p>';
					$books[] = <<<EOS
					<dt><a href="{$permalink}">{$asinlist->post_title}</a><br /><small>（投稿日: {$date}）</small></dt>
					<dd>{$display}</dd>
EOS;
	    		}
	    	}
			$heredoc .= '<dl>';
			foreach($books as $book){
				$heredoc .= $book;
			}
			$heredoc .= '</dl>';
	    } else {
	    	$heredoc = "<p>まだブログで書籍が紹介されていません。</p>\n";
	    }

		$output .= $heredoc;
		return $output;

	}

}


if(!function_exists('tmkm_amazon_view')) {
	/**
	 * Echo HTML strign with asin code.
	 * @global WpTmkmAmazonView $wpTmkmAmazonView
	 * @param string $asin ASIN code
	 * @return void
	 */
	function tmkm_amazon_view($asin) {
		global $wpTmkmAmazonList;
		$wpTmkmAmazonList->amazon_view($asin);
	}
}

/**
 * Enqueue Stylesheet
 * @global type $tmkm_plugin_directory 
 */
function add_tmkmamazon_stylesheet(){
	global $tmkm_plugin_directory;
	wp_enqueue_style('wp-tmkm-amazon', $tmkm_plugin_directory.'tmkm-amazon.css', array(), '2.01');
}
add_action('wp_print_styles', 'add_tmkmamazon_stylesheet');



/******************************************************************************
 * WpTmkmAmazon WordPress Plugin Class & Funtcion Define
 *****************************************************************************/

$wpTmkmAmazonList =  new WpTmkmAmazonList();
$wpTmkmAmazonAdmin =  new WpTmkmAmazonAdmin();